/* 
   PXKDrawingEngine.h

   The drawing engine encapsulates all windowing system
   dependencies.  This leads to a small performance drop,
   and to a substantial gain in reusability for the other
   elements of the kit.

   Reusing the other objects of the kit with other windowing
   systems should be only a matter of rewriting these functions.

   The drawing code is composed of a series of functions instead
   of being grouped in a class, because function calls are
   faster than method calls, and when we're drawing, speed is
   the name of the game.

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author:  Pascal Forget <pascal@wsc.com>
   Date: January 1996
   
   This file is part of the GNUstep GUI X/DPS Backend.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#ifndef _GNUstep_H_PXKDrawingEngine
#define _GNUstep_H_PXKDrawingEngine

#include <AppKit/NSColor.h>
#include <AppKit/NSView.h>
#include <AppKit/NSWindow.h>

/*
 * initializeDrawingSystem() is called once from the NSApplication object
 * when the application starts
 */

int initializeDrawingEngine(void);

void load_font(XFontStruct **font_info);

/* Creating and destroying views and windows */

Window createXWindow(NSRect rect);
int createView(NSView *sender, int superviewNum);
int destroyView(int viewNum);

void displayView(int viewNum);
void displayWindow(int windowNum);
void drawViewBackground(int viewNum);

void setViewFrame(int viewNum, NSRect frameRect);
void setViewFrameOrigin(int viewNum, NSPoint newOrigin);
void setViewFrameRotation(int viewNum, float angle);
void setViewFrameSize(int viewNum, NSSize newSize);

void setWindowFrame(int windowNum, NSRect frameRect);
void setWindowFrameOrigin(int windowNum, NSPoint newOrigin);
void setWindowFrameSize(int windowNum, NSSize newSize);
void setWindowTitle(int windowNum, NSString *title);

/* Utility functions not part of OpenStep */

void closeWindow(int windowNum);
void setCurrentColor(NSColor *aColor);

/* Functions to be called by other elements of the drawing engine */

int indexOfViewNumber(int viewNum);
int indexOfWindowNumber(int windowNum);
int indexOfXWindowRepresentingAView(Window xWindow);
int indexOfXWindowRepresentingAWindow(Window xWindow);
NSWindow *nswindow_for_xwindow(Window xWindow);
int numberForXWindow(Window win);

/* These are called from NSApplication */

void miniaturizeAll(id sender);
void preventWindowOrdering(void);
void updateWindows(void);

void pxkDrawLine(NSPoint a, NSPoint b, NSColor *color);
void pxkDrawLineRect(NSRect line, NSColor *color);
void pxkFillRect(NSRect rect, NSColor *color);

/* Application Kit Functions */

/* Rectangle Drawing Functions */

void NSEraseRect(NSRect aRect);          /* fills the rect with white.       */
void NSHighlightRect(NSRect aRect);      /* LTGRAY->WHITE, and vice-versa.   */
void NSRectClip(NSRect aRect);           /* Intersects the current clip path */
void NSRectClipList(const NSRect *rects, int count);
//void NSRectFill(NSRect aRect);           /* Fills rect using current color   */
void NSRectFillList(const NSRect *rects, int count);

/*
 * NSRectFillWithGrays() Fills each rectangle in the array 'rects'
 * with the gray whose value is stored at the corresponding location
 * in the array 'grays'.  Both arrays must be count elements long.
 * Avoid overlapping rectangles, because the drawing order can't be
 * guaranteed.
 */

void NSRectFillListWithGrays(const NSRect *rects,const float *grays,int count);


/* Private function */
NSPoint intersectionPoint(NSRect lineA, NSRect lineB);
float dist2D(NSPoint a, NSPoint b);
NSRect clipLineToRect(NSRect aLine, NSRect aRect);

/* Draw a Bordered Rectangle */

void NSDrawButton(NSRect aRect, NSRect clipRect);    /* Bordered ltgray rect */
void NSDrawGrayBezel(NSRect aRect, NSRect clipRect); /* pushed-in button     */
void NSDrawGroove(NSRect aRect, NSRect clipRect);    /* like a Box           */

#if 0
void NSDrawTiledRects(NSRect boundsRect,
		      NSRect clipRect,
		      const NSRectEdge *sides,
		      const float *grays,
		      int count);
#endif

void NSDrawWhiteBezel(NSRect aRect, NSRect clipRect); /* like a TextField    */
void NSFrameRect(NSRect aRect); /* Draws a frame of width 1.0 use curr color */
void NSFrameRectWithWidth(NSRect aRect, float frameWidth);

Display *
xDisplay(void); 

#endif





